/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Swedish Institute
 *      of Computer Science and its contributors.
 * 4. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: test_mem.c,v 1.5 2001/06/21 08:32:11 adam Exp $
 */

#include <stdio.h>
#include <unistd.h>

#include "lwip/debug.h"
#include "lwip/def.h"
#include "lwip/mem.h"
#include "lwip/sys.h"
#include "lwip/stats.h"

static void main_thread(void *arg);

#define SIZE 100

/*-----------------------------------------------------------------------------------*/
int
main(int argc, char **argv)
{
  stats_init();
  sys_init();
  mem_init();
  sys_thread_new((void *)(main_thread), NULL);
  pause();
  exit(0);
}
/*-----------------------------------------------------------------------------------*/
static void
main_thread(void *arg)
{
  char *mem[SIZE];
  int i;


  /* Allocate memory. */  
  for(i = 0; i < SIZE; i++) {
    mem[i] = mem_malloc(i * 4);
    if(mem[i] == NULL) {
      printf("Allocation failed for i = %d\n", i);
    } else {
      printf("Successful allocation for i = %d (%p)\n", i, mem[i]);
    }
  }

  /* Deallocate the memory. */
  for(i = 0; i < SIZE; i++) {
    mem_free(mem[i]);
  }

  /* Allocate memory. */  
  for(i = 0; i < SIZE; i++) {
    mem[i] = mem_malloc(i * 4);
    if(mem[i] == NULL) {
      printf("allocation failed for i = %d\n", i);
    } else {
      printf("Successful allocation for i = %d (%p)\n", i, mem[i]);
    }
  }

  /* Deallocate the memory. */
  for(i = SIZE - 1; i >= 0; i--) {
    mem_free(mem[i]);
  }

  
  /* Allocate memory. */  
  for(i = 0; i < SIZE; i++) {
    mem[i] = mem_malloc(i * 8);
    if(mem[i] == NULL) {
      printf("allocation failed for i = %d\n", i);
    } else {
      printf("Successful allocation for i = %d (%p)\n", i, mem[i]);
    }
  }

  /* Deallocate the memory. */
  for(i = SIZE - 1; i >= 0; i -= 2) {
    mem_free(mem[i]);
    mem[i] = NULL;
  }
  for(i = SIZE - 2; i >= 0; i -= 2) {
    mem_free(mem[i]);
    mem[i] = NULL;
  }


  /* Allocate memory. */  
  for(i = 0; i < SIZE; i += 2) {
    mem[i] = mem_malloc(i * 1);
    if(mem[i] == NULL) {
      printf("allocation failed for i = %d\n", i);
    } else {
      printf("Successful allocation for i = %d (%p)\n", i, mem[i]);
    }
  }
    
  for(i = 1; i < SIZE; i += 2) {
    mem[i] = mem_malloc(i * 1);
    if(mem[i] == NULL) {
      printf("allocation failed for i = %d\n", i);
    } else {
      printf("Successful allocation for i = %d (%p)\n", i, mem[i]);
    }
  }

  /* Deallocate the memory. */
  for(i = 0; i < SIZE; i++) {
    mem_free(mem[i]);
  }

  exit(0);
}
/*-----------------------------------------------------------------------------------*/
